function CRMAssistentesController(ScreenService, templateManager, CRM_Utilities) {
    
    this.openPopupNewRow = function() {
        var widgetNewRow = templateManager.container.getWidget('popupNewRow');
        
        ScreenService.openPopup(widgetNewRow);
        widgetNewRow.newRow();
        widgetNewRow.activate();
    };
    
    this.openPopupNewRowFromView = function(widget) {
        var popup = templateManager.container.getWidget('popupNewRow');
        var mainRow = templateManager.container.getWidget('assistencia').currentRow;
        var operPrincipalField = popup.getField('NMOPERPRINCIPAL');
        
        popup.parent = widget;
        this.openPopupNewRow();
        
        popup.currentRow.NROPERPRINCIPAL = mainRow.NRPARCNEGOCIO;
        popup.currentRow.NMOPERPRINCIPAL = mainRow.NMOPERADOR;
        operPrincipalField.readOnly = true;
        
        this.updateNewAssitFilter(popup);
    };
    
    this.openPopupNewRowFromList = function() {
        var popup = templateManager.container.getWidget('popupNewRow');
        var mainRow = templateManager.container.getWidget('assistencia');
        var operPrincipalField = popup.getField('NMOPERPRINCIPAL');
        
        popup.parent = mainRow;
        this.openPopupNewRow();
        
        operPrincipalField.readOnly = false;
    };
    
    this.updateNewAssitFilter = function(widget) {
        var assistField = widget.getField('NROPERASSIST');
        
        var assistOperFilter = assistField.dataSourceFilter.find(x =>
            x.name == 'NROPERPRINCIPAL');
        
        assistField.readOnly = Util.isEmptyOrBlank(widget.currentRow.NROPERPRINCIPAL);
        
        assistOperFilter.value = widget.currentRow.NROPERPRINCIPAL;
    };
    
    this.saveNewAssist = function(widget) {
        if(!widget.isValid()) {
            return;
        }
        
        var row = widget.currentRow;
        
        var params = {
            requestType: 'Row',
            row: row
        };
        
        CRM_Utilities.post('/saveNewAssist', params).then(response => {
            this.showMessageAfterSaveAndReload(widget, response).then(() => {
                this.openSwipeIfIsNew(widget);
            });
        });
    };
    
    this.openSwipeIfIsNew = function(popup) {
        if(popup.getParent().name !== 'assistencia') {
            return;
        }
        
        var mainWid = templateManager.container.getWidget('assistencia');
        var nrOperPrincipal = popup.currentRow.NROPERPRINCIPAL;
        
        mainWid.currentRow = mainWid.dataSource.data.find(row => {
            return row.NRPARCNEGOCIO == nrOperPrincipal;
        });
        
        this.openAssistList(mainWid);
    };
    
    this.reloadParentAndMainWidget = function(popup) {
        var mainWid = templateManager.container.getWidget('assistencia');
        var parentWid = popup.getParent();
        var reloadPromise;
        
        if(mainWid.name == parentWid.name) {
            reloadPromise = mainWid.reload();
        } else {
            reloadPromise = this.reloadListAndView();
        }
        
        return reloadPromise;
    };
    
    this.openAssistList = function(widget) {
        var tabWidget = templateManager.container.getWidget('tabAssist');
        var listWidget = templateManager.container.getWidget('assistentes');
        
        var listFilter = CRM_Utilities.getDataSourceFilter(listWidget, 'NROPERPRINCIPAL');
        listFilter.value = widget.currentRow.NRPARCNEGOCIO;
        
        ScreenService.openSwipe(tabWidget);
        listWidget.activate();
        
        listWidget.unSelectAll();
        
        listWidget.reload();
    };
    
    this.reloadListAndView = function() {
        var listWid = templateManager.container.getWidget('assistencia');
        var viewWid = templateManager.container.getWidget('assistentes');
        
        var promise1 = listWid.reload();
        var promise2 = viewWid.reload();
        
        return Promise.all([promise1, promise2]);
    };
    
    this.cancelPopup = function(popupWidget) {
        var parentWid = popupWidget.getParent();
        
        ScreenService.closePopup(popupWidget);
        parentWid.activate();
    };
    
    this.askToRemoveAssist = function(widget) {
        var msg = 'Deseja realmente remover esse assistente?';
        
        ScreenService.confirmMessage(msg, 'question', () => {
            this.removeAssist(widget);
        });
    };
    
    this.removeAssist = function(widget) {
        var checkedAssists = widget.dataSource.checkedRows;
        
        var row = {
            ASSISTENCIAS: checkedAssists
        };
        
        var params = {
            requestType: 'Row',
            row: row
        };
        
        CRM_Utilities.post('/removeAssist', params).then(response => {
            widget.unSelectAll();
            this.showMessageAfterRemoveAndReload(response).then(() => {
                if(widget.dataSource.data.length == 0) {
                    ScreenService.closeSwipe();
                }
            });
        });
    };
    
    this.showMessageAfterRemoveAndReload = function(response) {
        var removed = response.dataset.DADOS.REMOVED;
        var notRemoved = response.dataset.DADOS.NOT_REMOVED;
        
        var message = '';
        
        if(notRemoved.length > 0) {
            message = notRemoved.length > 1 ?
                'Os seguintes assistentes não puderam ser removidos:<br>' :
                'O seguinte assistente não pôde ser removido:<br>';
            
            message += notRemoved.reduce(function(concat, assist) {
                return concat + '<br>-> ' + assist.NMOPERADOR;
            }, '');
            
            message += '<br><br>Entre em contato com um administrador do sistema.';
        } else {
            message = (removed.length > 1 ?
                'Assistentes removidos ' : 'Assistente removido ') +
                'com sucesso!';
        }
        
        var promiseReload = this.reloadListAndView();
        ScreenService.showMessage(message);
        
        return promiseReload;
    };
    
    this.showMessageAfterSaveAndReload = function(widget, response) {
        var added = response.dataset.DADOS.ADDED;
        var notAdded = response.dataset.DADOS.NOT_ADDED;
        var assistField = widget.getField('NROPERASSIST');
        
        var message = '';
        
        if(notAdded.length > 0) {
            message = notAdded.length > 1 ?
                'Os seguintes assistentes não puderam ser adicionados:<br>' :
                'O seguinte assistente não pôde ser adicionado:<br>';
            
            message += notAdded.reduce(function(concat, assist) {
                return concat + '<br>-> ' + assist.NMOPERADOR;
            }, '');
            
            message += '<br><br>Entre em contato com um administrador do sistema.';
            
            assistField.reload();
        } else {
            message = (added.length > 1 ?
                'Assistentes adicionados ' : 'Assistente adicionado ') +
                'com sucesso!';
                
            ScreenService.closePopup();
            widget.getParent().activate();
        }
        
        var promiseReload = this.reloadParentAndMainWidget(widget);
        ScreenService.showMessage(message);
        
        return promiseReload;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register("CRMAssistentesController", CRMAssistentesController);
});