/* Tecnisa - Tecnologia em Informtica

 RAISERROR('', 16,1);

*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
                O.NAME = 'CONSULTA_ALIQUOTA'  AND
	        TYPE = 'P' 		      AND
	        U.UID = O.UID                 AND
	        U.NAME = USER)
DROP PROCEDURE CONSULTA_ALIQUOTA
GO

CREATE PROCEDURE CONSULTA_ALIQUOTA
                 @P_IMPOSTO     VARCHAR(02),
                 @P_ENTRSAIDAL  VARCHAR(01),
                 @P_EMPRESA     VARCHAR(02),
                 @P_PRODUTO     VARCHAR(20),
                 @P_INSCESTA    VARCHAR(16),
                 @P_PAISORIG    VARCHAR(04),
                 @P_ESTAORIG    VARCHAR(02),
                 @P_PAISDEST    VARCHAR(04),
                 @P_ESTADEST    VARCHAR(02),
                 @P_MUNIORIG    VARCHAR(04),
                 @P_BASECALC    NUMERIC(15,2),
                 @P_INIVALID    DATETIME,
                 @P_FILIAL      VARCHAR(08),
                 @P_LANCTONF    VARCHAR(10),
                 @P_CDGRUPOFISC VARCHAR(03),
                 @P_CDTIPOOPER  VARCHAR(02) = NULL,
                 @P_IDSISTEMA   VARCHAR(03) = NULL,
                 @P_LANCPRODUTO VARCHAR(01) = NULL,           --Exclusivo Unificao
                 @P_VRPRODUTO   DOUBLE PRECISION = NULL,      --Exclusivo Unificao
                 @P_CDFILIRECE  VARCHAR(04) = NULL,           --Exclusivo Unificao
                 @P_CDFORNECED  VARCHAR(21) = NULL,           --Exclusivo Unificao
                 @P_CDCLIENTE   VARCHAR(21) = NULL,           --Exclusivo Unificao
                 @P_IDORIGEMPROD VARCHAR(01) = 'N'



AS
   DECLARE @V_SEQUENCIA  VARCHAR(02); /*Varavel para gerar o sequencial da alquota*/
   DECLARE @V_CDARVPROD  VARCHAR(20); /*Cdigo de rvore do produto(Alquota por nvel ou todos)*/

   DECLARE @V_UTUFORIG   CHAR(1);     /*Utiliza UF de origem*/
   DECLARE @V_UTUFDEST   CHAR(1);     /*Utiliza UF de destino*/
   DECLARE @V_UTMUNIORIG CHAR(1);     /*Utiliza municipio (origem ou destino)*/
   DECLARE @V_UTFAIXVLR  CHAR(1);     /*Utiliza faixa de valores*/
   DECLARE @V_IDUTALIQTPOPER CHAR(1); /*Utiliza alquota por tipo de operao*/
   DECLARE @V_IDPRODNOTA VARCHAR(01); /*Visualizao do imposto: C-No corpo da nota, P-No produto*/

   DECLARE @V_VRITEMPROD DOUBLE PRECISION; /*Valor lquido do item*/
   DECLARE @V_CONTADOR   INTEGER;          /*Varavel para o total de alquotas encontradas*/

   /*VARIVEIS DO CURSOR ALIQUOTA*/
   DECLARE @V_ALIQ_NRALIQUOTA   VARCHAR(10);
   DECLARE @V_ALIQ_VRPERREDBC   NUMERIC(10,5);
   DECLARE @V_ALIQ_VR1PERREDBC  NUMERIC(10,5);
   DECLARE @V_ALIQ_VRPARDEDUZ   NUMERIC(15,2);
   DECLARE @V_ALIQ_VRALIQUOTA   NUMERIC(10,2);
   DECLARE @V_ALIQ_DSMENSALIQU  VARCHAR(120);
   DECLARE @V_ALIQ_IDTRATIMPST  VARCHAR(01);
   DECLARE @V_ALIQ_VRPBCSUBTRIB DOUBLE PRECISION;
   DECLARE @V_ALIQ_IDPMCBASECAL VARCHAR(01);
   DECLARE @V_ALIQ_CDTIPOOPER   VARCHAR(02);
   DECLARE @V_ALIQ_CDMENSAGEM   VARCHAR(03);
   DECLARE @V_ALIQ_VRADICALIQFECP DOUBLE PRECISION;

   DECLARE @V_CDGRUPOFISC VARCHAR(03); /*Cdigo do grupo fiscal*/
   DECLARE @V_CDTIPOOPER  VARCHAR(02); /*Cdigo do tipo de operao*/
   DECLARE @V_EXISTE      INTEGER;
   DECLARE @V_COUNTMENS   INTEGER;
   DECLARE @V_DSMENSALIQU VARCHAR(120);
   DECLARE @V_COUNT       INTEGER;

   /*VARIVEIS DO CURSOR C_REGRA*/
   DECLARE @V_REGRA_VRPERREDBC DOUBLE PRECISION;
   DECLARE @V_REGRA_VRPARDEDUZ DOUBLE PRECISION;
   DECLARE @V_REGRA_VRALIQUOTA DOUBLE PRECISION;
   DECLARE @V_REGRA_IDTRATIMPST VARCHAR(01);
   DECLARE @V_REGRA_VRPBCSUBTRIB DOUBLE PRECISION;
   DECLARE @V_REGRA_IDPMCBASECA VARCHAR(01);
   DECLARE @V_REGRA_VR1PERREDBC DOUBLE PRECISION;
   DECLARE @V_REGRA_CDREGRAFISC VARCHAR(10);
   DECLARE @V_REGRA_CDFISCOPER     VARCHAR(10);
   DECLARE @V_REGRA_IDDIFEALIQ     VARCHAR(01);
   DECLARE @V_REGRA_IDDIFALIQCUSPR VARCHAR(01);
   DECLARE @V_REGRA_VRADICALIQFECP DOUBLE PRECISION;

   DECLARE @V_MUNIORIG VARCHAR(04);
   DECLARE @V_MUNIDEST VARCHAR(04);

   DECLARE @V_ALIQSIMPLES   INTEGER;
   DECLARE @V_VRALIQSIMPLES NUMERIC(10,2);

BEGIN
   /*Parametros do Imposto */
   SELECT @V_UTUFORIG   = ISNULL(MAX(IDUTUFORIG),'N'),   /*Utiliza UF de origem*/
          @V_UTUFDEST   = ISNULL(MAX(IDUTUFDEST),'N'),   /*Utiliza UF de destino*/
          @V_UTMUNIORIG = ISNULL(MAX(IDUTMUNIORIG),'N'), /*Utiliza municipio (Origem ou destino)*/
          @V_UTFAIXVLR  = ISNULL(MAX(IDUTFAIXVLR),'N'),  /*Utiliza faixa de valores*/
          @V_IDPRODNOTA = MAX(IDPRODNOTA),               /*Visualizao do imposto (C-Corpo da nota; P-Produto)*/
          @V_IDUTALIQTPOPER = ISNULL(MAX(IDUTALIQTPOPER),'N') /*Utiliza aliquota por tipo de operao*/
   FROM IMPOSTO
   WHERE (CDIMPOSTO = @P_IMPOSTO);

   IF @V_UTMUNIORIG = 'O' BEGIN
      SET @V_MUNIORIG = @P_MUNIORIG;
      SET @V_MUNIDEST = NULL;
   END;
   IF @V_UTMUNIORIG = 'D' BEGIN
      SET @V_MUNIORIG = NULL;
      SET @V_MUNIDEST = @P_MUNIORIG;
   END;
   IF @V_UTMUNIORIG = 'N' BEGIN
      SET @V_MUNIORIG = NULL;
      SET @V_MUNIDEST = NULL;
   END;


   IF (@P_LANCTONF <> '0000000000') BEGIN /*Se a consulta_alquota for chamada a partir de um lanamento*/
      IF (@V_UTFAIXVLR = 'S') AND (@V_IDPRODNOTA = 'P') AND (@P_ENTRSAIDAL = 'S') BEGIN
         IF @P_IDSISTEMA = 'PED' BEGIN
            SELECT @V_VRITEMPROD = SUM((VRPREUNITPED * QTITEMPED) - ((VRPREUNITPED * QTITEMPED) * (VRDESITPED/100)))
              FROM ITEMPEDIDO
             WHERE CDFILIAL = SUBSTRING(@P_FILIAL,1,4) AND
                   NRPEDIDO = SUBSTRING(@P_FILIAL,5,10);
         END
         ELSE BEGIN
            SELECT @V_VRITEMPROD = SUM(VRTOTAITEM - VRDESCITEM)
              FROM ITEMPRODUTO
             WHERE CDFILIAL   = @P_FILIAL AND
                   NRLANCTONF = @P_LANCTONF;
         END;
      END
      ELSE BEGIN
         IF(@P_LANCPRODUTO = 'S') BEGIN
            SET @V_VRITEMPROD = @P_VRPRODUTO;
         END
         ELSE BEGIN
            SET @V_VRITEMPROD = @P_BASECALC;
         END;
      END;
   END
   ELSE BEGIN /*Se a Consulta_alquota no for chamada a partir de um lanamento*/
      IF(@P_LANCPRODUTO = 'S') BEGIN
         SET @V_VRITEMPROD = @P_VRPRODUTO;
      END
      ELSE BEGIN
         SET @V_VRITEMPROD = @P_BASECALC;
      END;
   END;

  	SET @V_SEQUENCIA = '01';

   SET @V_CDGRUPOFISC = NULL;

   /* Verificao das aliquotas referente as filiais parametrizadas na tela Parmetros de Grupo Fiscal (Imposto/Inscrio Estadual) */
      /*Por Fornecedor*/

   SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
     FROM GRFISIMPIEFORN G, (SELECT CDFORNECED
                               FROM NOTAFISCAL
                              WHERE CDFILIAL = @P_FILIAL
                                AND NRLANCTONF = @P_LANCTONF
                                AND @P_CDFORNECED IS NULL
                              UNION ALL
                             SELECT CDFORNECED
                               FROM FORNECEDOR
                              WHERE CDFORNECED = @P_CDFORNECED) F
    WHERE G.CDIMPOSTO = @P_IMPOSTO
      AND G.CDFORNECED = F.CDFORNECED
      AND G.CDEMPRESA = @P_EMPRESA
      AND G.CDINSCESTA = @P_INSCESTA;

   IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/

      DECLARE C_REGRA CURSOR FOR
         SELECT VRPERREDBC, VRPARDEDUZ, VRALIQUOTA, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECA,
                VR1PERREDBC, CDREGRAFISC, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP
           FROM TEKNISA.CONSULTA_REGRA_FISCAL(@P_IMPOSTO, @P_EMPRESA, @V_CDGRUPOFISC, @P_ENTRSAIDAL, @P_CDTIPOOPER, @P_INSCESTA, @P_PRODUTO, @P_INIVALID,
                                              @P_PAISORIG, @P_ESTAORIG, @V_MUNIORIG, @P_PAISDEST, @P_ESTADEST, @V_MUNIDEST, @P_BASECALC, 'N', @P_IDORIGEMPROD)
          WHERE VRALIQUOTA IS NOT NULL;
      OPEN C_REGRA;
      FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                   @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;
      WHILE (@@FETCH_STATUS = 0) BEGIN

         SELECT @V_EXISTE = COUNT(*)
           FROM ALIQTEMP
          WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF;

         IF (@V_EXISTE > 0) BEGIN
            DELETE FROM ALIQTEMP
            WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF;
         END;

         SET @V_ALIQSIMPLES = 0;

         SELECT @V_VRALIQSIMPLES = VRALIQUOTA,
                @V_ALIQSIMPLES   = COUNT(*)
           FROM ALIQSIMPNAC
          WHERE CDFORNECED  = @P_CDFORNECED
            AND CDREGRAFISC = @V_REGRA_CDREGRAFISC
            AND CDIMPOSTO   = @P_IMPOSTO
          GROUP BY VRALIQUOTA;

         IF (@V_ALIQSIMPLES > 0) BEGIN
            INSERT INTO ALIQTEMP
               (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
            VALUES
               (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, 0, 0, @V_VRALIQSIMPLES, NULL,
                @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, 0, @V_REGRA_CDREGRAFISC,
                @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, 'N', 'N', @V_REGRA_VRADICALIQFECP);
         END
         ELSE BEGIN
            INSERT INTO ALIQTEMP
               (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
            VALUES
               (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, NULL,
                @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC,
                @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP);
         END;

         SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);
         FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                      @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;

      END;
      CLOSE C_REGRA;
      DEALLOCATE C_REGRA;
   END
   ELSE BEGIN
      /*Por Cliente*/
      SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
        FROM GRFISIMPIECLIE G, (SELECT CDCLIENTE
                                  FROM NOTAFISCAL
                                 WHERE CDFILIAL = @P_FILIAL
                                   AND NRLANCTONF = @P_LANCTONF
                                   AND @P_CDCLIENTE IS NULL
                                 UNION ALL
                                SELECT CDCLIENTE
                                  FROM CLIENTE
                                 WHERE CDCLIENTE = @P_CDCLIENTE) F
       WHERE G.CDIMPOSTO = @P_IMPOSTO
         AND G.CDCLIENTE = F.CDCLIENTE
         AND G.CDEMPRESA = @P_EMPRESA
         AND G.CDINSCESTA = @P_INSCESTA;

      IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/
         DECLARE C_REGRA CURSOR FOR
            SELECT VRPERREDBC, VRPARDEDUZ, VRALIQUOTA, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECA,
                   VR1PERREDBC, CDREGRAFISC, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP
              FROM TEKNISA.CONSULTA_REGRA_FISCAL(@P_IMPOSTO, @P_EMPRESA, @V_CDGRUPOFISC, @P_ENTRSAIDAL, @P_CDTIPOOPER, @P_INSCESTA, @P_PRODUTO, @P_INIVALID,
                                                 @P_PAISORIG, @P_ESTAORIG, @V_MUNIORIG, @P_PAISDEST, @P_ESTADEST, @V_MUNIDEST, @P_BASECALC, 'N', @P_IDORIGEMPROD)
             WHERE VRALIQUOTA IS NOT NULL;
         OPEN C_REGRA;
         FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                      @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;
         WHILE (@@FETCH_STATUS = 0) BEGIN

            SELECT @V_EXISTE = COUNT(*)
              FROM ALIQTEMP
             WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF;

            IF (@V_EXISTE > 0) BEGIN
               DELETE FROM ALIQTEMP
               WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF;
            END;

            SET @V_ALIQSIMPLES = 0;

            SELECT @V_VRALIQSIMPLES = VRALIQUOTA,
                   @V_ALIQSIMPLES   = COUNT(*)
              FROM ALIQSIMPNAC
             WHERE CDFORNECED  = @P_CDFORNECED
               AND CDREGRAFISC = @V_REGRA_CDREGRAFISC
               AND CDIMPOSTO   = @P_IMPOSTO
             GROUP BY VRALIQUOTA;

            IF (@V_ALIQSIMPLES > 0) BEGIN
               INSERT INTO ALIQTEMP
                  (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                   IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                   CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
               VALUES
                  (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, 0, 0, @V_VRALIQSIMPLES, NULL,
                   @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, 0, @V_REGRA_CDREGRAFISC,
                   @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, 'N', 'N', @V_REGRA_VRADICALIQFECP);
            END
            ELSE BEGIN      
               INSERT INTO ALIQTEMP
                  (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                   IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                   CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
               VALUES
                  (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, NULL,
                   @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC,
                   @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP);
            END;

            SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);
            FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                         @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;

         END;
         CLOSE C_REGRA;
         DEALLOCATE C_REGRA;
      END
      ELSE BEGIN

         SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
           FROM GRUPOFISCIMPFIL G, (SELECT (CASE @P_ENTRSAIDAL WHEN 'E' THEN CDFILIRECE ELSE CDFILIEMIT END) AS CDFILIAL
                                      FROM NOTAFISCAL
                                     WHERE CDFILIAL = @P_FILIAL
                                       AND NRLANCTONF = @P_LANCTONF
                                       AND @P_IDSISTEMA IS NULL
                                       AND @P_CDFILIRECE IS NULL
                                     UNION ALL
                                    SELECT CDFILIAL
                                      FROM PEDIDO
                                     WHERE CDFILIAL = SUBSTRING(@P_FILIAL,1,4)
                                       AND NRPEDIDO = SUBSTRING(@P_FILIAL,5,10)
                                       AND @P_IDSISTEMA = 'PED'
                                     UNION ALL
                                    SELECT CDFILIAL
                                      FROM FILIAL
                                     WHERE CDFILIAL = @P_CDFILIRECE) F

          WHERE G.CDIMPOSTO = @P_IMPOSTO
            AND G.CDFILIAL = F.CDFILIAL;

         IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/
            DECLARE C_REGRA CURSOR FOR
               SELECT VRPERREDBC, VRPARDEDUZ, VRALIQUOTA, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECA,
                      VR1PERREDBC, CDREGRAFISC, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP
                 FROM TEKNISA.CONSULTA_REGRA_FISCAL(@P_IMPOSTO, @P_EMPRESA, @V_CDGRUPOFISC, @P_ENTRSAIDAL, @P_CDTIPOOPER, @P_INSCESTA, @P_PRODUTO, @P_INIVALID,
                                                    @P_PAISORIG, @P_ESTAORIG, @V_MUNIORIG, @P_PAISDEST, @P_ESTADEST, @V_MUNIDEST, @P_BASECALC, 'N', @P_IDORIGEMPROD)
                WHERE VRALIQUOTA IS NOT NULL;
            OPEN C_REGRA;
            FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                         @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;
            WHILE (@@FETCH_STATUS = 0) BEGIN

                SELECT @V_EXISTE = COUNT(*)
                  FROM ALIQTEMP
                 WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF;

                IF (@V_EXISTE > 0) BEGIN
                   DELETE FROM ALIQTEMP
                   WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF;
                END;

                SET @V_ALIQSIMPLES = 0;

                SELECT @V_VRALIQSIMPLES = VRALIQUOTA,
                       @V_ALIQSIMPLES   = COUNT(*)
                  FROM ALIQSIMPNAC
                 WHERE CDFORNECED  = @P_CDFORNECED
                   AND CDREGRAFISC = @V_REGRA_CDREGRAFISC
                   AND CDIMPOSTO   = @P_IMPOSTO
                 GROUP BY VRALIQUOTA;

                IF (@V_ALIQSIMPLES > 0) BEGIN
                   INSERT INTO ALIQTEMP
                      (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                       IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                       CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
                   VALUES
                      (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, 0, 0, @V_VRALIQSIMPLES, NULL,
                       @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, 0, @V_REGRA_CDREGRAFISC,
                       @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, 'N', 'N', @V_REGRA_VRADICALIQFECP);
                END
                ELSE BEGIN    
                   INSERT INTO ALIQTEMP
                      (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                       IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                       CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
                   VALUES
                      (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, NULL,
                       @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC,
                       @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP);
                END;

                SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);
                FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                         @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;

            END;
            CLOSE C_REGRA;
            DEALLOCATE C_REGRA;
         END
         ELSE BEGIN
            DECLARE C_REGRA CURSOR FOR
               SELECT VRPERREDBC, VRPARDEDUZ, VRALIQUOTA, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECA,
                      VR1PERREDBC, CDREGRAFISC, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP
                 FROM TEKNISA.CONSULTA_REGRA_FISCAL(@P_IMPOSTO, @P_EMPRESA, @V_CDGRUPOFISC, @P_ENTRSAIDAL, @P_CDTIPOOPER, @P_INSCESTA, @P_PRODUTO, @P_INIVALID,
                                                    @P_PAISORIG, @P_ESTAORIG, @V_MUNIORIG, @P_PAISDEST, @P_ESTADEST, @V_MUNIDEST, @P_BASECALC, 'N', @P_IDORIGEMPROD)
                WHERE VRALIQUOTA IS NOT NULL;
            OPEN C_REGRA;
            FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                            @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;
            WHILE (@@FETCH_STATUS = 0) BEGIN

               SELECT @V_EXISTE = COUNT(*)
                 FROM ALIQTEMP
                 WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF
                   AND CDREGRATMP IS NULL;

               IF (@V_EXISTE > 0) BEGIN
                  DELETE FROM ALIQTEMP
                  WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF
                    AND CDREGRATMP IS NULL;
               END;

               SET @V_ALIQSIMPLES = 0;

               SELECT @V_VRALIQSIMPLES = VRALIQUOTA,
                      @V_ALIQSIMPLES   = COUNT(*)
                 FROM ALIQSIMPNAC
                WHERE CDFORNECED  = @P_CDFORNECED
                  AND CDREGRAFISC = @V_REGRA_CDREGRAFISC
                  AND CDIMPOSTO   = @P_IMPOSTO
                GROUP BY VRALIQUOTA;

               IF (@V_ALIQSIMPLES > 0) BEGIN
                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                      IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                      CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, 0, 0, @V_VRALIQSIMPLES, NULL,
                      @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, 0, @V_REGRA_CDREGRAFISC,
                      @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, 'N', 'N', @V_REGRA_VRADICALIQFECP);
               END
               ELSE BEGIN
                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP, VRALIQUTMP, DSMENSALITMP,
                      IDTRIMPSTTMP, VRPBCSTTMP, IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, CDREGRATMP,
                      CDTIPOOPERTMP, CDFISCOPER, IDDIFEALIQ, IDDIFALIQCUSPR, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, NULL,
                      @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,NULL, @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC,
                      @P_CDTIPOOPER, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP);

               END;

               SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);
               FETCH NEXT FROM C_REGRA INTO @V_REGRA_VRPERREDBC, @V_REGRA_VRPARDEDUZ, @V_REGRA_VRALIQUOTA, @V_REGRA_IDTRATIMPST, @V_REGRA_VRPBCSUBTRIB, @V_REGRA_IDPMCBASECA,
                                            @V_REGRA_VR1PERREDBC, @V_REGRA_CDREGRAFISC, @V_REGRA_CDFISCOPER, @V_REGRA_IDDIFEALIQ, @V_REGRA_IDDIFALIQCUSPR, @V_REGRA_VRADICALIQFECP;

            END;
            CLOSE C_REGRA;
            DEALLOCATE C_REGRA;
         END;
      END;
   END;

   SET @V_EXISTE = 0;
   SELECT @V_EXISTE = COUNT(*)
     FROM ALIQTEMP
    WHERE NRALIQTEMP  = @P_FILIAL+@P_LANCTONF
      AND CDREGRATMP IS NOT NULL;

   IF NOT(@V_EXISTE > 0) BEGIN


      SET @V_CONTADOR = 0;

      IF @P_CDGRUPOFISC IS NOT NULL BEGIN /*Se h grupo fiscal*/
         IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
            SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER), @V_CONTADOR = COUNT(*)
            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
            WHERE (CDIMPOSTO    = @P_IMPOSTO)                            AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                         AND
                  (CDEMPRESA    = @P_EMPRESA)                            AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    AND
                  (CDGRUPOFISC  = @P_CDGRUPOFISC)                        AND
                  ((CDPAISORIG  = @P_PAISORIG)                           AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     AND
                  ((CDPAISDEST  = @P_PAISDEST)                           AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            IF @V_CDTIPOOPER IS NOT NULL BEGIN /*Se possui alquota por tipo de operao*/
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                             AND
                     (IDENTRSAIDAL = @P_ENTRSAIDAL)                          AND
                     (CDEMPRESA    = @P_EMPRESA)                             AND
                     (CDINSCESTA   = RTRIM(@P_INSCESTA))                     AND
                     (CDGRUPOFISC  = @P_CDGRUPOFISC)                         AND
                     ((CDPAISORIG  = @P_PAISORIG)                            AND
                     (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')       AND
                     ((CDPAISDEST  = @P_PAISDEST)                            AND
                     (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')       AND
                     ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                     ((VRINIFAIXA  < @V_VRITEMPROD)                          AND
                     (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')   AND
                     (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N') AND
                     (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            END
            ELSE IF @V_CDTIPOOPER IS NULL BEGIN
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                            AND
                     (IDENTRSAIDAL = @P_ENTRSAIDAL)                         AND
                     (CDEMPRESA    = @P_EMPRESA)                            AND
                     (CDINSCESTA   = RTRIM(@P_INSCESTA))                    AND
                     (CDGRUPOFISC  = @P_CDGRUPOFISC)                        AND
                     ((CDPAISORIG  = @P_PAISORIG)                           AND
                     (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     AND
                     ((CDPAISDEST  = @P_PAISDEST)                           AND
                     (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     AND
                     ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                     ((VRINIFAIXA  < @V_VRITEMPROD)                         AND
                     (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                     (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                     (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);
            END;
         END
         ELSE BEGIN /*Se imposto da nota*/
            SELECT @V_CDARVPROD = MAX(A.CDPRODUTO),
                   @V_CONTADOR = COUNT(*),
                   @V_CDTIPOOPER = MAX(CDTIPOOPER)
            FROM ALIQUOTA A
            WHERE (CDIMPOSTO    = @P_IMPOSTO)
              AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
              AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
              AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
              AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
              AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
              AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
              AND (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
              AND (A.CDPRODUTO IS NULL);
         END;

         IF @V_CONTADOR > 0 BEGIN/*Se encontrou alquota para o grupo fiscal*/
            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN/*Se encontrou alquota para o grupo fiscal*/

                  DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER,
                         CDMENSAGEM, VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE	(A.CDIMPOSTO = @P_IMPOSTO) AND
                           (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                           (A.CDEMPRESA = @P_EMPRESA) AND
                           (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                           (A.CDGRUPOFISC = @P_CDGRUPOFISC) AND
                           (P.CDARVPROD = @V_CDARVPROD) AND
                           ((A.CDPAISORIG = @P_PAISORIG) AND
                           (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                           ((A.CDPAISDEST = @P_PAISDEST) AND
                           (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                           ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                           ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                           (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                           (CDTIPOOPER = @V_CDTIPOOPER) AND
                           (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                              FROM ALIQUOTA B LEFT JOIN PRODUTO P ON (B.CDPRODUTO = P.CDPRODUTO)
                                             WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                   (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                   (CDEMPRESA = @P_EMPRESA) AND
                                                   (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                   (CDGRUPOFISC = @P_CDGRUPOFISC) AND
                                                   (P.CDARVPROD = @V_CDARVPROD) AND
                                                   ((CDPAISORIG = @P_PAISORIG) AND
                                                   (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                   ((CDPAISDEST = @P_PAISDEST) AND
                                                   (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                   ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                   ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                   (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                   (CDTIPOOPER = @V_CDTIPOOPER) AND
                                                   (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,  @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                       (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                        VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                        IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP, 
                        VRADICALIQFECP)
                     VALUES
                       (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                        @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                        @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                        @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                     @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER,
                         CDMENSAGEM, VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO) AND
                         (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                         (A.CDEMPRESA = @P_EMPRESA) AND
                         (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                         (A.CDGRUPOFISC = @P_CDGRUPOFISC) AND
                         (A.CDPRODUTO IS NULL) AND
                         ((A.CDPAISORIG = @P_PAISORIG) AND
                          (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                         ((A.CDPAISDEST = @P_PAISDEST) AND
                          (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                         ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                         ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                          (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                         (CDTIPOOPER = @V_CDTIPOOPER) AND
                         (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                   (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                   (CDEMPRESA = @P_EMPRESA) AND
                                                   (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                   (CDGRUPOFISC = @P_CDGRUPOFISC) AND
                                                   (CDPRODUTO IS NULL) AND
                                                   ((CDPAISORIG = @P_PAISORIG) AND
                                                    (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                   ((CDPAISDEST = @P_PAISDEST) AND
                                                    (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                   ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                   ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                    (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                   (CDTIPOOPER = @V_CDTIPOOPER) AND
                                                   (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP, 
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                         @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                     @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto de maior nvel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                         CDMENSAGEM, VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE	(A.CDIMPOSTO = @P_IMPOSTO) AND
                           (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                           (A.CDEMPRESA = @P_EMPRESA) AND
                           (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                           (A.CDGRUPOFISC = @P_CDGRUPOFISC) AND
                           (P.CDARVPROD = @V_CDARVPROD) AND
                           ((A.CDPAISORIG = @P_PAISORIG) AND
                            (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                           ((A.CDPAISDEST = @P_PAISDEST) AND
                            (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                           ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                           ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                            (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                           (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                           (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                              FROM ALIQUOTA B LEFT JOIN PRODUTO P ON (B.CDPRODUTO = P.CDPRODUTO)
                                             WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                   (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                   (CDEMPRESA = @P_EMPRESA) AND
                                                   (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                   (CDGRUPOFISC = @P_CDGRUPOFISC) AND
                                                   (P.CDARVPROD = @V_CDARVPROD) AND
                                                   ((CDPAISORIG = @P_PAISORIG) AND
                                                    (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                   ((CDPAISDEST = @P_PAISDEST) AND
                                                    (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                   ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                   ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                    (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                   (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                                                   (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                   SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                          CDMENSAGEM, VRADICALIQFECP
                     FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                    WHERE	(A.CDIMPOSTO = @P_IMPOSTO) AND
                           (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                           (A.CDEMPRESA = @P_EMPRESA) AND
                           (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                           (A.CDGRUPOFISC = @P_CDGRUPOFISC) AND
                           (A.CDPRODUTO IS NULL) AND
                           ((A.CDPAISORIG = @P_PAISORIG) AND
                            (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                           ((A.CDPAISDEST = @P_PAISDEST) AND
                            (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                           ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                           ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                            (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                           (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                           (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                              FROM ALIQUOTA
                                             WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                   (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                   (CDEMPRESA = @P_EMPRESA) AND
                                                   (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                   (CDGRUPOFISC = @P_CDGRUPOFISC) AND
                                                   (CDPRODUTO IS NULL) AND
                                                   ((CDPAISORIG = @P_PAISORIG) AND
                                                    (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                   ((CDPAISDEST = @P_PAISDEST) AND
                                                    (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                   ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                   ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                    (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                   (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                                                   (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END;
         END;
      END
      ELSE BEGIN /*Se no h grupo fiscal*/
         SELECT @V_CDTIPOOPER = MAX(A.CDTIPOOPER)
           FROM ALIQUOTA A LEFT JOIN PRODUTO P
                           ON (A.CDPRODUTO = P.CDPRODUTO)
          WHERE (CDIMPOSTO    = @P_IMPOSTO)
            AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
            AND (CDEMPRESA    = @P_EMPRESA)
            AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
            AND (CDGRUPOFISC IS NULL)
            AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
            AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
            AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
            AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
            AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
            AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);

         IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
            SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
              FROM ALIQUOTA A LEFT JOIN PRODUTO P
                              ON (A.CDPRODUTO = P.CDPRODUTO)
             WHERE (CDIMPOSTO    = @P_IMPOSTO)
               AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
               AND (CDGRUPOFISC IS NULL)
               AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
               AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
               AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
               AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
               AND (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
               AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
         END
         ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
            SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
              FROM ALIQUOTA A LEFT JOIN PRODUTO P
                              ON (A.CDPRODUTO = P.CDPRODUTO)
             WHERE (CDIMPOSTO    = @P_IMPOSTO)
               AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
               AND (CDGRUPOFISC IS NULL)
               AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
               AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
               AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
               AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
               AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
               AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
         END;

         IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
               DECLARE C_ALIQUOTA CURSOR FOR
                SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER, CDMENSAGEM,
                       VRADICALIQFECP
                  FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                  LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                 WHERE	(A.CDIMPOSTO = @P_IMPOSTO) AND
                        (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                        (A.CDEMPRESA = @P_EMPRESA) AND
                        (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                        (A.CDGRUPOFISC IS NULL) AND
                        (P.CDARVPROD = @V_CDARVPROD) AND
                        ((A.CDPAISORIG = @P_PAISORIG) AND
                         (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                        ((A.CDPAISDEST = @P_PAISDEST) AND
                         (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                        ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                        ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                         (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                        (CDTIPOOPER = @V_CDTIPOOPER) AND
                        (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                           FROM ALIQUOTA B LEFT JOIN PRODUTO P ON (B.CDPRODUTO = P.CDPRODUTO)
                                          WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                (CDEMPRESA = @P_EMPRESA) AND
                                                (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                (CDGRUPOFISC IS NULL) AND
                                                (P.CDARVPROD = @V_CDARVPROD) AND
                                                ((CDPAISORIG = @P_PAISORIG) AND
                                                 (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                ((CDPAISDEST = @P_PAISDEST) AND
                                                 (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                 (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                (CDTIPOOPER = @V_CDTIPOOPER) AND
                                                (DTINIVALID <= @P_INIVALID)));
               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               DECLARE C_ALIQUOTA CURSOR FOR
                SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                       VRADICALIQFECP
                  FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                  LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                 WHERE (A.CDIMPOSTO = @P_IMPOSTO) AND
                       (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                       (A.CDEMPRESA = @P_EMPRESA) AND
                       (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                       (A.CDGRUPOFISC IS NULL) AND
                       (P.CDARVPROD = @V_CDARVPROD) AND
                       ((A.CDPAISORIG = @P_PAISORIG) AND
                        (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                       ((A.CDPAISDEST = @P_PAISDEST) AND
                        (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                       ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                       ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                        (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                       (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                       (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                          FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                         WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                (CDEMPRESA = @P_EMPRESA) AND
                                                (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                (CDGRUPOFISC IS NULL) AND
                                                (P.CDARVPROD = @V_CDARVPROD) AND
                                                ((CDPAISORIG = @P_PAISORIG) AND
                                                 (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                ((CDPAISDEST = @P_PAISDEST) AND
                                                 (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                 (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                                                (DTINIVALID <= @P_INIVALID)));
               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END
         ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para "todos"*/
            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se h alquota por tipo de operao*/
               DECLARE C_ALIQUOTA CURSOR FOR
                SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER, CDMENSAGEM,
                       VRADICALIQFECP
                  FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                 WHERE (A.CDIMPOSTO = @P_IMPOSTO) AND
                       (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                       (A.CDEMPRESA = @P_EMPRESA) AND
                       (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                       (A.CDGRUPOFISC IS NULL) AND
                       (A.CDPRODUTO IS NULL) AND
                       ((A.CDPAISORIG = @P_PAISORIG) AND
                        (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                       ((A.CDPAISDEST = @P_PAISDEST) AND
                        (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                       ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                       ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                        (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                       (CDTIPOOPER = @V_CDTIPOOPER) AND
                       (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                          FROM ALIQUOTA
                                         WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                (CDEMPRESA = @P_EMPRESA) AND
                                                (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                (CDGRUPOFISC IS NULL) AND
                                                (CDPRODUTO IS NULL) AND
                                                ((CDPAISORIG = @P_PAISORIG) AND
                                                 (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                ((CDPAISDEST = @P_PAISDEST) AND
                                                 (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                 (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                (CDTIPOOPER = @V_CDTIPOOPER) AND
                                                (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               DECLARE C_ALIQUOTA CURSOR FOR
                SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                       VRADICALIQFECP
                  FROM ALIQUOTA A LEFT JOIN MENSALIQU ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                 WHERE (A.CDIMPOSTO = @P_IMPOSTO) AND
                       (A.IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                       (A.CDEMPRESA = @P_EMPRESA) AND
                       (A.CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                       (A.CDGRUPOFISC IS NULL) AND
                       (A.CDPRODUTO IS NULL) AND
                       ((A.CDPAISORIG = @P_PAISORIG) AND
                        (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                       ((A.CDPAISDEST = @P_PAISDEST) AND
                        (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                       ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                       ((A.VRINIFAIXA < @V_VRITEMPROD) AND
                        (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                       (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                       (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                          FROM ALIQUOTA
                                         WHERE	(CDIMPOSTO = @P_IMPOSTO) AND
                                                (IDENTRSAIDAL = @P_ENTRSAIDAL) AND
                                                (CDEMPRESA = @P_EMPRESA) AND
                                                (CDINSCESTA = RTRIM(@P_INSCESTA)) AND
                                                (CDGRUPOFISC IS NULL) AND
                                                (CDPRODUTO IS NULL) AND
                                                ((CDPAISORIG = @P_PAISORIG) AND
                                                 (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N') AND
                                                ((CDPAISDEST = @P_PAISDEST) AND
                                                 (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N') AND
                                                ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                                                ((VRINIFAIXA < @V_VRITEMPROD) AND
                                                 (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') AND
                                                (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                                                (DTINIVALID <= @P_INIVALID)));
               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END;
      END;
      /* Verificao das aliquotas referente as filiais parametrizadas na tela Parmetros de Grupo Fiscal (Imposto/Filial) */
      SET @V_CDGRUPOFISC = NULL;

      SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
        FROM GRUPOFISCIMPFIL G, (SELECT (CASE @P_ENTRSAIDAL WHEN 'E' THEN CDFILIRECE ELSE CDFILIEMIT END) AS CDFILIAL
                                   FROM NOTAFISCAL
                                  WHERE CDFILIAL = @P_FILIAL
                                    AND NRLANCTONF = @P_LANCTONF
                                    AND @P_IDSISTEMA IS NULL
                                    AND @P_CDFILIRECE IS NULL
                                 UNION ALL
                                 SELECT CDFILIAL
                                   FROM PEDIDO
                                  WHERE CDFILIAL = SUBSTRING(@P_FILIAL,1,4)
                                    AND NRPEDIDO = SUBSTRING(@P_FILIAL,5,10)
                                    AND @P_IDSISTEMA = 'PED'
                                  UNION ALL
                                 SELECT CDFILIAL
                                   FROM FILIAL
                                  WHERE CDFILIAL = @P_CDFILIRECE) F
       WHERE G.CDIMPOSTO = @P_IMPOSTO
         AND G.CDFILIAL = F.CDFILIAL;

      IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/
         IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
            SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER),
                   @V_CONTADOR = COUNT(*)
            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
            WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            IF @V_CDTIPOOPER IS NOT NULL BEGIN /*Se possui alquota por tipo de operao*/
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')  AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            END
            ELSE IF @V_CDTIPOOPER IS NULL BEGIN
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);
            END;
         END
         ELSE BEGIN /*Se imposto da nota*/
            SELECT @V_CDARVPROD = MAX(A.CDPRODUTO),
                   @V_CONTADOR = COUNT(*),
                   @V_CDTIPOOPER = MAX(CDTIPOOPER)
            FROM ALIQUOTA A
            WHERE (CDIMPOSTO    = @P_IMPOSTO)
              AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
              AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
              AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
              AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
              AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
              AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
              AND (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
              AND (A.CDPRODUTO IS NULL);
         END;

         IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               DECLARE C_ALIQUOTA CURSOR FOR
                SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDTIPOOPER, CDMENSAGEM,
                       VRADICALIQFECP
                  FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                  LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                 WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                   AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                   AND (A.CDEMPRESA = @P_EMPRESA)
                   AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                   AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                   AND (P.CDARVPROD = @V_CDARVPROD)
                   AND ((A.CDPAISORIG = @P_PAISORIG)
                   AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                   AND ((A.CDPAISDEST = @P_PAISDEST)
                   AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                   AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                   AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                   AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                   AND (CDTIPOOPER = @V_CDTIPOOPER)
                   AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                          FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                         WHERE (CDIMPOSTO = @P_IMPOSTO)
                                           AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                           AND (CDEMPRESA = @P_EMPRESA)
                                           AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                           AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                           AND (P.CDARVPROD = @V_CDARVPROD)
                                           AND ((CDPAISORIG = @P_PAISORIG)
                                           AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                           AND ((CDPAISDEST = @P_PAISDEST)
                                           AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                           AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                           AND ((VRINIFAIXA < @V_VRITEMPROD)
                                           AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                           AND (CDTIPOOPER = @V_CDTIPOOPER)
                                           AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN

                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP, 
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (A.CDPRODUTO IS NULL)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER)
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (CDPRODUTO IS NULL)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CDTIPOOPER = @V_CDTIPOOPER)
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END
         ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (P.CDARVPROD = @V_CDARVPROD)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (P.CDARVPROD = @V_CDARVPROD)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (A.CDPRODUTO IS NULL)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (CDPRODUTO IS NULL)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END;
      END;
      /* Verificao das aliquotas referente aos Fornecedores/Clientes parametrizadas na tela Parmetros de Grupo Fiscal (Imposto/Inscrio Estadual) */
      /*Por Fornecedor*/
      SET @V_CDGRUPOFISC = NULL;

      SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
        FROM GRFISIMPIEFORN G, (SELECT CDFORNECED
                                  FROM NOTAFISCAL
                                 WHERE CDFILIAL = @P_FILIAL
                                   AND NRLANCTONF = @P_LANCTONF
                                   AND @P_CDFORNECED IS NULL
                                 UNION ALL
                                SELECT CDFORNECED
                                  FROM FORNECEDOR
                                 WHERE CDFORNECED = @P_CDFORNECED) F
       WHERE G.CDIMPOSTO = @P_IMPOSTO
         AND G.CDFORNECED = F.CDFORNECED
         AND G.CDEMPRESA = @P_EMPRESA
         AND G.CDINSCESTA = @P_INSCESTA;

      IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/
         IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
            SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER),
                   @V_CONTADOR = COUNT(*)
            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
            WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            IF @V_CDTIPOOPER IS NOT NULL BEGIN /*Se possui alquota por tipo de operao*/
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')  AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            END
            ELSE IF @V_CDTIPOOPER IS NULL BEGIN
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);
            END;
         END
         ELSE BEGIN /*Se imposto da nota*/
            SELECT @V_CDARVPROD = MAX(A.CDPRODUTO),
                   @V_CONTADOR = COUNT(*),
                   @V_CDTIPOOPER = MAX(CDTIPOOPER)
            FROM ALIQUOTA A
            WHERE (CDIMPOSTO    = @P_IMPOSTO)
              AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
              AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
              AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
              AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
              AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
              AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
              AND (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
              AND (A.CDPRODUTO IS NULL);
         END;

         IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDTIPOOPER, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (P.CDARVPROD = @V_CDARVPROD)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER)
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (P.CDARVPROD = @V_CDARVPROD)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CDTIPOOPER = @V_CDTIPOOPER)
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN

                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (A.CDPRODUTO IS NULL)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER)
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (CDPRODUTO IS NULL)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CDTIPOOPER = @V_CDTIPOOPER)
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END
         ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN  /*Se no possui alquota por tipo de operao*/
            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (P.CDARVPROD = @V_CDARVPROD)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (P.CDARVPROD = @V_CDARVPROD)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (A.CDPRODUTO IS NULL)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (CDPRODUTO IS NULL)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL,@V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END;
      END;
         /*Por Cliente*/
      SET @V_CDGRUPOFISC = NULL;

      SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
        FROM GRFISIMPIECLIE G, (SELECT CDCLIENTE
                                  FROM NOTAFISCAL
                                 WHERE CDFILIAL = @P_FILIAL
                                   AND NRLANCTONF = @P_LANCTONF
                                   AND @P_CDCLIENTE IS NULL
                                 UNION ALL
                                SELECT CDCLIENTE
                                  FROM CLIENTE
                                 WHERE CDCLIENTE = @P_CDCLIENTE) F
       WHERE G.CDIMPOSTO = @P_IMPOSTO
         AND G.CDCLIENTE = F.CDCLIENTE
         AND G.CDEMPRESA = @P_EMPRESA
         AND G.CDINSCESTA = @P_INSCESTA;

      IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/
         IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
            SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER),
                   @V_CONTADOR = COUNT(*)
            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
            WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            IF @V_CDTIPOOPER IS NOT NULL BEGIN /*Se possui alquota por tipo de operao*/
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')  AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);

            END
            ELSE IF @V_CDTIPOOPER IS NULL BEGIN
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
               FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
               WHERE (CDIMPOSTO    = @P_IMPOSTO)                           AND
                  (IDENTRSAIDAL = @P_ENTRSAIDAL)                        	AND
                  (CDEMPRESA    = @P_EMPRESA)                            	AND
                  (CDINSCESTA   = RTRIM(@P_INSCESTA))                    	AND
                  (CDGRUPOFISC  = @V_CDGRUPOFISC)                        	AND
                  ((CDPAISORIG  = @P_PAISORIG)                           	AND
                  (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')     	AND
                  ((CDPAISDEST  = @P_PAISDEST)                           	AND
                  (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')     	AND
                  ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N')) AND
                  ((VRINIFAIXA  < @V_VRITEMPROD)                         	AND
                  (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N') 	AND
                  (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N') AND
                  (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')),ISNULL(@P_PRODUTO,'_')) = 1);
            END;
         END
         ELSE BEGIN /*Se imposto da nota*/
            SELECT @V_CDARVPROD = MAX(A.CDPRODUTO),
                   @V_CONTADOR = COUNT(*),
                   @V_CDTIPOOPER = MAX(CDTIPOOPER)
            FROM ALIQUOTA A
            WHERE (CDIMPOSTO    = @P_IMPOSTO)
              AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
              AND (CDEMPRESA    = @P_EMPRESA)
              AND (CDINSCESTA   = RTRIM(@P_INSCESTA))
              AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
              AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
              AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
              AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
              AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
              AND (ISNULL(CDTIPOOPER,@P_CDTIPOOPER) = @P_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
              AND (A.CDPRODUTO IS NULL);
         END;
         IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDTIPOOPER,
                         CDMENSAGEM, VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (P.CDARVPROD = @V_CDARVPROD)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER)
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (P.CDARVPROD = @V_CDARVPROD)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CDTIPOOPER = @V_CDTIPOOPER)
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER,  @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN

                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, A.CDTIPOOPER, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (A.CDPRODUTO IS NULL)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER)
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (CDPRODUTO IS NULL)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CDTIPOOPER = @V_CDTIPOOPER)
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDTIPOOPER,  @V_ALIQ_CDMENSAGEM,
                                               @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, CDTIPOOPERTMP, VR1PREDBCTMP, CDINSCESTAATMP,
                      VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, @V_ALIQ_VR1PERREDBC, @P_INSCESTA,
                      @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END
         ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                    LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (P.CDARVPROD = @V_CDARVPROD)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA A LEFT JOIN PRODUTO P ON (A.CDPRODUTO = P.CDPRODUTO)
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (P.CDARVPROD = @V_CDARVPROD)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
               DECLARE C_ALIQUOTA CURSOR FOR
                  SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA, DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL, CDMENSAGEM,
                         VRADICALIQFECP
                    FROM ALIQUOTA A LEFT JOIN MENSALIQU M ON (A.NRALIQUOTA = M.NRALIQUOTA)
                   WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                     AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (A.CDEMPRESA = @P_EMPRESA)
                     AND (A.CDINSCESTA = RTRIM(@P_INSCESTA))
                     AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                     AND (A.CDPRODUTO IS NULL)
                     AND ((A.CDPAISORIG = @P_PAISORIG)
                     AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((A.CDPAISDEST = @P_PAISDEST)
                     AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                     AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                     AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                            FROM ALIQUOTA
                                           WHERE (CDIMPOSTO = @P_IMPOSTO)
                                             AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                             AND (CDEMPRESA = @P_EMPRESA)
                                             AND (CDINSCESTA = RTRIM(@P_INSCESTA))
                                             AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                             AND (CDPRODUTO IS NULL)
                                             AND ((CDPAISORIG = @P_PAISORIG)
                                             AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                             AND ((CDPAISDEST = @P_PAISDEST)
                                             AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                             AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                             AND ((VRINIFAIXA < @V_VRITEMPROD)
                                             AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                             AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                             AND (DTINIVALID <= @P_INIVALID)));

               OPEN C_ALIQUOTA;
               FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                               @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                               @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                               @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                               @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

               WHILE (@@FETCH_STATUS = 0) BEGIN
                  SELECT @V_EXISTE = COUNT(*)
                    FROM ALIQTEMP
                   WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                  IF @V_EXISTE > 0 BEGIN
                     DELETE FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                  END;

                  SELECT @V_COUNTMENS = COUNT(*)
                    FROM MENSAGEM
                   WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                     AND IDSITUACAOMENS = 'I';

                  IF @V_COUNTMENS > 0 BEGIN
                     SET @V_DSMENSALIQU = NULL;
                  END
                  ELSE BEGIN
                     SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                  END;

                  INSERT INTO ALIQTEMP
                     (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                      VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                      IDPMCBSCALTMP, VR1PREDBCTMP, CDINSCESTAATMP, VRADICALIQFECP)
                  VALUES
                     (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                      @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                      @V_ALIQ_IDPMCBASECAL, @V_ALIQ_VR1PERREDBC, @P_INSCESTA, @V_ALIQ_VRADICALIQFECP);

                  SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
               END;
               CLOSE C_ALIQUOTA;
               DEALLOCATE C_ALIQUOTA;
            END;
         END;
      END;
      /*------------------------IMPOSTOS FEDERAIS--------------------------------------*/
      SELECT @V_COUNT = COUNT(*)
        FROM ALIQTEMP
       WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

      IF @V_COUNT = 0 BEGIN
         IF @P_CDGRUPOFISC IS NOT NULL BEGIN /*Se h grupo fiscal*/
            IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
               SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER), @V_CONTADOR = COUNT(*)
                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);

               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA   IS NULL)
                     AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA   IS NULL)
                     AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END;
            END
            ELSE BEGIN/*Se imposto da nota*/
               SELECT @V_CDARVPROD = MAX(A.CDPRODUTO), @V_CONTADOR = COUNT(*), @V_CDTIPOOPER = MAX(CDTIPOOPER)
                 FROM ALIQUOTA A
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @P_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (A.CDPRODUTO IS NULL);
            END;

            IF @V_CONTADOR > 0 BEGIN /*Se encontrou alquota para o grupo fiscal*/
               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
                  IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto de maior nvel*/
                     DECLARE C_ALIQUOTA CURSOR FOR
                        SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                               DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                               A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                          FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                          ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                          LEFT JOIN PRODUTO P
                                          ON (A.CDPRODUTO = P.CDPRODUTO)
                         WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                           AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                           AND (A.CDEMPRESA = @P_EMPRESA)
                           AND (A.CDINSCESTA IS NULL)
                           AND (A.CDGRUPOFISC = @P_CDGRUPOFISC)
                           AND (P.CDARVPROD = @V_CDARVPROD)
                           AND ((A.CDPAISORIG = @P_PAISORIG)
                           AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                           AND ((A.CDPAISDEST = @P_PAISDEST) AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                           AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                           AND ((A.VRINIFAIXA < @V_VRITEMPROD) AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                           AND (CDTIPOOPER = @V_CDTIPOOPER)
                           AND (A.DTINIVALID = ( SELECT MAX(DTINIVALID)
                                                   FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                   ON (A.CDPRODUTO = P.CDPRODUTO)
                                                  WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                    AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                    AND (CDEMPRESA = @P_EMPRESA)
                                                    AND (CDINSCESTA IS NULL)
                                                    AND (CDGRUPOFISC = @P_CDGRUPOFISC)
                                                    AND (P.CDARVPROD = @V_CDARVPROD)
                                                    AND ((CDPAISORIG = @P_PAISORIG)
                                                    AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                    AND ((CDPAISDEST = @P_PAISDEST) AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                    AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                    AND ((VRINIFAIXA < @V_VRITEMPROD) AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                    AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                    AND (DTINIVALID <= @P_INIVALID)));

                     OPEN C_ALIQUOTA;
                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDTIPOOPER,  @V_ALIQ_CDMENSAGEM,
                                                     @V_ALIQ_VRADICALIQFECP;
                     WHILE (@@FETCH_STATUS = 0) BEGIN
                        SELECT @V_COUNTMENS = COUNT(*)
                          FROM MENSAGEM
                         WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                           AND IDSITUACAOMENS = 'I';

                        IF @V_COUNTMENS > 0 BEGIN
                           SET @V_DSMENSALIQU = NULL;
                        END
                        ELSE BEGIN
                           SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                        END;

                        INSERT INTO ALIQTEMP
                           (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                            VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                            IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP, 
                            VRADICALIQFECP)
                        VALUES
                           (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                            @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                            @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC, 
                            @V_ALIQ_VRADICALIQFECP);

                        SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                        FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDTIPOOPER,  @V_ALIQ_CDMENSAGEM,
                                                     @V_ALIQ_VRADICALIQFECP;
                     END;
                     CLOSE C_ALIQUOTA;
                     DEALLOCATE C_ALIQUOTA;
                  END
                  ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                     DECLARE C_ALIQUOTA CURSOR FOR
                        SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                               DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                               A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                          FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                          ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                         WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                           AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                           AND (A.CDEMPRESA = @P_EMPRESA)
                           AND (A.CDINSCESTA IS NULL)
                           AND (A.CDGRUPOFISC = @P_CDGRUPOFISC)
                           AND (A.CDPRODUTO IS NULL)
                           AND ((A.CDPAISORIG = @P_PAISORIG)
                           AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                           AND ((A.CDPAISDEST = @P_PAISDEST) AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                           AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                           AND ((A.VRINIFAIXA < @V_VRITEMPROD) AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                           AND (CDTIPOOPER = @V_CDTIPOOPER)
                           AND (A.DTINIVALID = ( SELECT MAX(DTINIVALID)
                                                   FROM ALIQUOTA
                                                  WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                    AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                    AND (CDEMPRESA = @P_EMPRESA)
                                                    AND (CDINSCESTA IS NULL)
                                                    AND (CDGRUPOFISC = @P_CDGRUPOFISC)
                                                    AND (CDPRODUTO IS NULL)
                                                    AND ((CDPAISORIG = @P_PAISORIG)
                                                    AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                    AND ((CDPAISDEST = @P_PAISDEST)
                                                    AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                    AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                    AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                    AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                    AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                    AND (DTINIVALID <= @P_INIVALID)));
                     OPEN C_ALIQUOTA;
                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                     @V_ALIQ_VRADICALIQFECP;
                     WHILE (@@FETCH_STATUS = 0) BEGIN
                        SELECT @V_COUNTMENS = COUNT(*)
                          FROM MENSAGEM
                         WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                           AND IDSITUACAOMENS = 'I';

                        IF @V_COUNTMENS > 0 BEGIN
                           SET @V_DSMENSALIQU = NULL;
                        END
                        ELSE BEGIN
                           SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                        END;

                        INSERT INTO ALIQTEMP
                           (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                            VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                            IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                            VRADICALIQFECP)
                        VALUES
                           (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                            @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                            @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                            @V_ALIQ_VRADICALIQFECP);

                        SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                        FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDTIPOOPER,  @V_ALIQ_CDMENSAGEM,
                                                     @V_ALIQ_VRADICALIQFECP;
                     END;
                     CLOSE C_ALIQUOTA;
                     DEALLOCATE C_ALIQUOTA;
                  END;
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto de maior nvel*/
                     DECLARE C_ALIQUOTA CURSOR FOR
                         SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                                DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                                CDMENSAGEM, VRADICALIQFECP
                           FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                           ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                           LEFT JOIN PRODUTO P
                                           ON (A.CDPRODUTO = P.CDPRODUTO)
                          WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                            AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                            AND (A.CDEMPRESA = @P_EMPRESA)
                            AND (A.CDINSCESTA IS NULL)
                            AND (A.CDGRUPOFISC = @P_CDGRUPOFISC)
                            AND (P.CDARVPROD = @V_CDARVPROD)
                            AND ((A.CDPAISORIG = @P_PAISORIG)
                            AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                            AND ((A.CDPAISDEST = @P_PAISDEST) AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                            AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                            AND ((A.VRINIFAIXA < @V_VRITEMPROD) AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                            AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                            AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                                                   WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                     AND (CDEMPRESA = @P_EMPRESA)
                                                     AND (CDINSCESTA IS NULL)
                                                     AND (CDGRUPOFISC = @P_CDGRUPOFISC)
                                                     AND (P.CDARVPROD = @V_CDARVPROD)
                                                     AND ((CDPAISORIG = @P_PAISORIG)
                                                     AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                     AND ((CDPAISDEST = @P_PAISDEST) AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                     AND ((VRINIFAIXA < @V_VRITEMPROD) AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                     AND (DTINIVALID <= @P_INIVALID)));
                     OPEN C_ALIQUOTA;
                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                     WHILE (@@FETCH_STATUS = 0) BEGIN
                        SELECT @V_COUNTMENS = COUNT(*)
                          FROM MENSAGEM
                         WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                           AND IDSITUACAOMENS = 'I';

                        IF @V_COUNTMENS > 0 BEGIN
                           SET @V_DSMENSALIQU = NULL;
                        END
                        ELSE BEGIN
                           SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                        END;

                        INSERT INTO ALIQTEMP
                           (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                            VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                            IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                        VALUES
                           (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                            @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                            @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                        SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);
                        FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                     END;
                     CLOSE C_ALIQUOTA;
                     DEALLOCATE C_ALIQUOTA;
                  END
                  ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                     DECLARE C_ALIQUOTA CURSOR FOR
                        SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                               DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                               CDMENSAGEM, VRADICALIQFECP
                          FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                          ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                         WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                           AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                           AND (A.CDEMPRESA = @P_EMPRESA)
                           AND (A.CDINSCESTA IS NULL)
                           AND (A.CDGRUPOFISC = @P_CDGRUPOFISC)
                           AND (A.CDPRODUTO IS NULL)
                           AND ((A.CDPAISORIG = @P_PAISORIG)
                           AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                           AND ((A.CDPAISDEST = @P_PAISDEST) AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                           AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                           AND ((A.VRINIFAIXA < @V_VRITEMPROD) AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                           AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                           AND (A.DTINIVALID = ( SELECT MAX(DTINIVALID)
                                                   FROM ALIQUOTA
                                                  WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                    AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                    AND (CDEMPRESA = @P_EMPRESA)
                                                    AND (CDINSCESTA IS NULL)
                                                    AND (CDGRUPOFISC = @P_CDGRUPOFISC)
                                                    AND (CDPRODUTO IS NULL)
                                                    AND ((CDPAISORIG = @P_PAISORIG)
                                                    AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                    AND ((CDPAISDEST = @P_PAISDEST)
                                                    AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                    AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                    AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                    AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                    AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                    AND (DTINIVALID <= @P_INIVALID)));

                     OPEN C_ALIQUOTA;
                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDMENSAGEM,   @V_ALIQ_VRADICALIQFECP;
                     WHILE (@@FETCH_STATUS = 0) BEGIN

                        SELECT @V_COUNTMENS = COUNT(*)
                          FROM MENSAGEM
                         WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                           AND IDSITUACAOMENS = 'I';

                        IF @V_COUNTMENS > 0 BEGIN
                           SET @V_DSMENSALIQU = NULL;
                        END
                        ELSE BEGIN
                           SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                        END;

                        INSERT INTO ALIQTEMP
                           (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                            VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                            IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                        VALUES
                           (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                            @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                            @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                        SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                        FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                     @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                     @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                     @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                     @V_ALIQ_CDMENSAGEM,   @V_ALIQ_VRADICALIQFECP;
                     END;
                     CLOSE C_ALIQUOTA;
                     DEALLOCATE C_ALIQUOTA;
                  END;
               END;
            END;
         END
         ELSE BEGIN /*Se no h grupo fiscal*/
            SELECT @V_CDTIPOOPER = MAX(A.CDTIPOOPER)
              FROM ALIQUOTA A LEFT JOIN PRODUTO P
                              ON (A.CDPRODUTO = P.CDPRODUTO)
             WHERE (CDIMPOSTO    = @P_IMPOSTO)
               AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
               AND (CDEMPRESA    = @P_EMPRESA)
               AND (CDINSCESTA   IS NULL)
               AND (CDGRUPOFISC IS NULL)
               AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
               AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
               AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
               AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
               AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
               AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);

            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC IS NULL)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC IS NULL)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
            END;

            IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                     SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                            DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                            A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                       FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                       ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                       LEFT JOIN PRODUTO P
                                       ON (A.CDPRODUTO = P.CDPRODUTO)
                      WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                        AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                        AND (A.CDEMPRESA = @P_EMPRESA)
                        AND (A.CDINSCESTA IS NULL)
                        AND (A.CDGRUPOFISC IS NULL)
                        AND (P.CDARVPROD = @V_CDARVPROD)
                        AND ((A.CDPAISORIG = @P_PAISORIG)
                        AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                        AND ((A.CDPAISDEST = @P_PAISDEST)
                        AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                        AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                        AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                        AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                        AND (CDTIPOOPER = @V_CDTIPOOPER)
                        AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                ON (A.CDPRODUTO = P.CDPRODUTO)
                                               WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                 AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                 AND (CDEMPRESA = @P_EMPRESA)
                                                 AND (CDINSCESTA IS NULL)
                                                 AND (CDGRUPOFISC IS NULL)
                                                 AND (P.CDARVPROD = @V_CDARVPROD)
                                                 AND ((CDPAISORIG = @P_PAISORIG)
                                                 AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                 AND ((CDPAISDEST = @P_PAISDEST)
                                                 AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                 AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                 AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                 AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                 AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                 AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                       (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                        VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                        IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                        VRADICALIQFECP)
                     VALUES
                       (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                        @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                        @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC, 
                        @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                      SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                             DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                             CDMENSAGEM, VRADICALIQFECP
                        FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                        ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                                        LEFT JOIN PRODUTO P
                                        ON (A.CDPRODUTO = P.CDPRODUTO)
                       WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                         AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                         AND (A.CDEMPRESA = @P_EMPRESA)
                         AND (A.CDINSCESTA IS NULL)
                         AND (A.CDGRUPOFISC IS NULL)
                         AND (P.CDARVPROD = @V_CDARVPROD)
                         AND ((A.CDPAISORIG = @P_PAISORIG)
                         AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                         AND ((A.CDPAISDEST = @P_PAISDEST)
                         AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                         AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                         AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                         AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                         AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                         AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (CDGRUPOFISC IS NULL)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM,   @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                       (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                        VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                        IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                       (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                        @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                        @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM,   @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END
            ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para "todos"*/
               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se h alquota por tipo de operao*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                     SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                            DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                            A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                       FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                       ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                      WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                        AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                        AND (A.CDEMPRESA = @P_EMPRESA)
                        AND (A.CDINSCESTA IS NULL)
                        AND (A.CDGRUPOFISC IS NULL)
                        AND (A.CDPRODUTO IS NULL)
                        AND ((A.CDPAISORIG = @P_PAISORIG)
                        AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                        AND ((A.CDPAISDEST = @P_PAISDEST)
                        AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                        AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                        AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                        AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                        AND (CDTIPOOPER = @V_CDTIPOOPER)
                        AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                FROM ALIQUOTA
                                               WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                 AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                 AND (CDEMPRESA = @P_EMPRESA)
                                                 AND (CDINSCESTA IS NULL)
                                                 AND (CDGRUPOFISC IS NULL)
                                                 AND (CDPRODUTO IS NULL)
                                                 AND ((CDPAISORIG = @P_PAISORIG)
                                                 AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                 AND ((CDPAISDEST = @P_PAISDEST)
                                                 AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                 AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                 AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                 AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                 AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                 AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                         @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER,   @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                     SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                            DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                            CDMENSAGEM, VRADICALIQFECP
                       FROM ALIQUOTA A LEFT JOIN MENSALIQU
                                       ON (A.NRALIQUOTA = MENSALIQU.NRALIQUOTA)
                      WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                        AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                        AND (A.CDEMPRESA = @P_EMPRESA)
                        AND (A.CDINSCESTA IS NULL)
                        AND (A.CDGRUPOFISC IS NULL)
                        AND (A.CDPRODUTO IS NULL)
                        AND ((A.CDPAISORIG = @P_PAISORIG)
                        AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                        AND ((A.CDPAISDEST = @P_PAISDEST)
                        AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                        AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                        AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                        AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                        AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                        AND (A.DTINIVALID = ( SELECT MAX(DTINIVALID)
                                                FROM ALIQUOTA
                                               WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                 AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                 AND (CDEMPRESA = @P_EMPRESA)
                                                 AND (CDINSCESTA IS NULL)
                                                 AND (CDGRUPOFISC IS NULL)
                                                 AND (CDPRODUTO IS NULL)
                                                 AND ((CDPAISORIG = @P_PAISORIG)
                                                 AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                 AND ((CDPAISDEST = @P_PAISDEST)
                                                 AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                 AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                 AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                 AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                 AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                 AND (DTINIVALID <= @P_INIVALID)));

                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM,   @V_ALIQ_VRADICALIQFECP;
                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END;
         END;

         /* Verificao das aliquotas referente as filiais parametrizadas na tela Parmetros de Grupo Fiscal (Imposto/Filial) */

         SET @V_CDGRUPOFISC = NULL;

         SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
           FROM GRUPOFISCIMPFIL G, (SELECT (CASE @P_ENTRSAIDAL WHEN 'E' THEN CDFILIRECE ELSE CDFILIEMIT END) AS CDFILIAL
                                      FROM NOTAFISCAL
                                     WHERE CDFILIAL = @P_FILIAL
                                       AND NRLANCTONF = @P_LANCTONF
                                       AND @P_IDSISTEMA IS NULL
                                       AND @P_CDFILIRECE IS NULL
                                     UNION ALL
                                    SELECT CDFILIAL
                                      FROM PEDIDO
                                     WHERE CDFILIAL = SUBSTRING(@P_FILIAL,1,4)
                                       AND NRPEDIDO = SUBSTRING(@P_FILIAL,5,10)
                                       AND @P_IDSISTEMA = 'PED'
                                     UNION ALL
                                    SELECT CDFILIAL
                                      FROM FILIAL
                                     WHERE CDFILIAL = @P_CDFILIRECE) F
          WHERE G.CDIMPOSTO = @P_IMPOSTO
            AND G.CDFILIAL = F.CDFILIAL;

         IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Filial*/
            IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
               SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER), @V_CONTADOR = COUNT(*)
                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);

               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA   IS NULL)
                     AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                      AND (CDEMPRESA    = @P_EMPRESA)
                      AND (CDINSCESTA   IS NULL)
                      AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                      AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                      AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                      AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                      AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                      AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                      AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END;
            END
            ELSE BEGIN /*Se imposto da nota*/
               SELECT @V_CDARVPROD = MAX(A.CDPRODUTO), @V_CONTADOR = COUNT(*), @V_CDTIPOOPER = MAX(CDTIPOOPER)
                 FROM ALIQUOTA A
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (A.CDPRODUTO IS NULL);
            END;
            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                         LEFT JOIN PRODUTO P
                                         ON (A.CDPRODUTO = P.CDPRODUTO)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (P.CDARVPROD = @V_CDARVPROD)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CDTIPOOPER = @V_CDTIPOOPER)
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                         @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (A.CDPRODUTO IS NULL)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CDTIPOOPER = @V_CDTIPOOPER)
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (CDPRODUTO IS NULL)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP, 
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                         @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                         LEFT JOIN PRODUTO P
                                         ON (A.CDPRODUTO = P.CDPRODUTO)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (P.CDARVPROD = @V_CDARVPROD)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (A.CDPRODUTO IS NULL)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (CDPRODUTO IS NULL)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END;
         END;

         ----------------------
         /* Verificao das aliquotas referente as filiais parametrizadas na tela Parmetros de Grupo Fiscal (Imposto/Inscrio Estadual) */
         /*Por Fornecedor*/
         SET @V_CDGRUPOFISC = NULL;

         SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
           FROM GRFISIMPIEFORN G, (SELECT CDFORNECED
                                     FROM NOTAFISCAL
                                    WHERE CDFILIAL = @P_FILIAL
                                      AND NRLANCTONF = @P_LANCTONF
                                      AND @P_CDFORNECED IS NULL
                                    UNION ALL
                                   SELECT CDFORNECED
                                     FROM FORNECEDOR
                                    WHERE CDFORNECED = @P_CDFORNECED) F
          WHERE G.CDIMPOSTO = @P_IMPOSTO
            AND G.CDFORNECED = F.CDFORNECED
            AND G.CDEMPRESA = @P_EMPRESA
            AND G.CDINSCESTA = @P_INSCESTA;

         IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Inscrio Estadual*/
            IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
               SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER), @V_CONTADOR = COUNT(*)
                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);

               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA   IS NULL)
                     AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA   IS NULL)
                     AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END;
            END
            ELSE BEGIN /*Se imposto da nota*/
               SELECT @V_CDARVPROD = MAX(A.CDPRODUTO), @V_CONTADOR = COUNT(*), @V_CDTIPOOPER = MAX(CDTIPOOPER)
                 FROM ALIQUOTA A
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (A.CDPRODUTO IS NULL);
            END;

            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                         LEFT JOIN PRODUTO P
                                         ON (A.CDPRODUTO = P.CDPRODUTO)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (P.CDARVPROD = @V_CDARVPROD)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CDTIPOOPER = @V_CDTIPOOPER)
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                  AND (DTINIVALID <= @P_INIVALID)));

                 OPEN C_ALIQUOTA;
                 FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                 @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                 @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                 @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                 @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM, 
                                                 @V_ALIQ_VRADICALIQFECP;

                 WHILE (@@FETCH_STATUS = 0) BEGIN
                    SELECT @V_EXISTE = COUNT(*)
                      FROM ALIQTEMP
                     WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                    IF @V_EXISTE > 0 BEGIN
                       DELETE FROM ALIQTEMP
                       WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                    END;

                    SELECT @V_COUNTMENS = COUNT(*)
                      FROM MENSAGEM
                     WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                       AND IDSITUACAOMENS = 'I';

                    IF @V_COUNTMENS > 0 BEGIN
                       SET @V_DSMENSALIQU = NULL;
                    END
                    ELSE BEGIN
                       SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                    END;

                    INSERT INTO ALIQTEMP
                       (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                        VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                        IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP, 
                        VRADICALIQFECP)
                    VALUES
                       (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                        @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                        @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                        @V_ALIQ_VRADICALIQFECP);

                    SET @V_SEQUENCIA = RIGHT(REPLICATE('0',2)+RTRIM(CONVERT(VARCHAR, CONVERT(INT,@V_SEQUENCIA) + 1)),2);

                    FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                    @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (A.CDPRODUTO IS NULL)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CDTIPOOPER = @V_CDTIPOOPER)
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (CDPRODUTO IS NULL)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                         @V_ALIQ_VRADICALIQFECP);


                    FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                    @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                         LEFT JOIN PRODUTO P
                                         ON (A.CDPRODUTO = P.CDPRODUTO)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (P.CDARVPROD = @V_CDARVPROD)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN

                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (A.CDPRODUTO IS NULL)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (CDPRODUTO IS NULL)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END;
         END;
         /*Por Cliente*/
         SET @V_CDGRUPOFISC = NULL;

         SELECT @V_CDGRUPOFISC = MAX(G.CDGRUPOFISC)
           FROM GRFISIMPIECLIE G, (SELECT CDCLIENTE
                                     FROM NOTAFISCAL
                                    WHERE CDFILIAL = @P_FILIAL
                                      AND NRLANCTONF = @P_LANCTONF
                                      AND @P_CDCLIENTE IS NULL
                                    UNION ALL
                                   SELECT CDCLIENTE
                                     FROM CLIENTE
                                    WHERE CDCLIENTE = @P_CDCLIENTE) F
          WHERE G.CDIMPOSTO = @P_IMPOSTO
            AND G.CDCLIENTE = F.CDCLIENTE
            AND G.CDEMPRESA = @P_EMPRESA
            AND G.CDINSCESTA = @P_INSCESTA;

         IF NOT (@V_CDGRUPOFISC IS NULL) BEGIN /*Se h grupo fiscal na tela Parmetros de Grupo Fiscal na aba Imposto/Inscrio Estadual*/
            IF @V_IDPRODNOTA = 'P' BEGIN /*Se imposto do item*/
               SELECT @V_CDTIPOOPER = MAX(CDTIPOOPER), @V_CONTADOR = COUNT(*)
                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);

               IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA  IS NULL)
                     AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CDTIPOOPER = @V_CDTIPOOPER OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END
               ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
                  SELECT @V_CDARVPROD = MAX(P.CDARVPROD)
                    FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                    ON (A.CDPRODUTO = P.CDPRODUTO)
                   WHERE (CDIMPOSTO    = @P_IMPOSTO)
                     AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                     AND (CDEMPRESA    = @P_EMPRESA)
                     AND (CDINSCESTA   IS NULL)
                     AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                     AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                     AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                     AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                     AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                     AND (CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),ISNULL(@P_PRODUTO,'_')), ISNULL(@P_PRODUTO,'_')) = 1);
               END;
            END
            ELSE BEGIN /*Se imposto da nota*/
               SELECT @V_CDARVPROD = MAX(A.CDPRODUTO), @V_CONTADOR = COUNT(*), @V_CDTIPOOPER = MAX(CDTIPOOPER)
                 FROM ALIQUOTA A
                WHERE (CDIMPOSTO    = @P_IMPOSTO)
                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                  AND (CDEMPRESA    = @P_EMPRESA)
                  AND (CDINSCESTA   IS NULL)
                  AND (CDGRUPOFISC  = @V_CDGRUPOFISC)
                  AND ((CDPAISORIG  = @P_PAISORIG) AND (SGESTAORIG  = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                  AND ((CDPAISDEST  = @P_PAISDEST) AND (SGESTADEST  = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                  AND ((VRINIFAIXA  < @V_VRITEMPROD) AND (VRFIMFAIXA  >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER), ISNULL(@P_CDTIPOOPER,'_')), ISNULL(@P_CDTIPOOPER,'_')) = 1  OR @V_IDUTALIQTPOPER = 'N')
                  AND (A.CDPRODUTO IS NULL);
            END;

            IF NOT (@V_CDTIPOOPER IS NULL) BEGIN /*Se possui alquota por tipo de operao*/

               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                         LEFT JOIN PRODUTO P
                                         ON (A.CDPRODUTO = P.CDPRODUTO)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (P.CDARVPROD = @V_CDARVPROD)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CDTIPOOPER = @V_CDTIPOOPER)
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                         @V_ALIQ_VRADICALIQFECP);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                    @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              A.CDTIPOOPER, CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (A.CDPRODUTO IS NULL)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CDTIPOOPER = @V_CDTIPOOPER)
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (CDPRODUTO IS NULL)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CDTIPOOPER = @V_CDTIPOOPER)
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                  @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDTIPOOPERTMP, CDINSCESTAATMP, VR1PREDBCTMP,
                         VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, @V_ALIQ_CDTIPOOPER, NULL, @V_ALIQ_VR1PERREDBC,
                         @V_ALIQ_VRADICALIQFECP);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDTIPOOPER, @V_ALIQ_CDMENSAGEM,
                                                    @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END
            ELSE IF (@V_CDTIPOOPER IS NULL) BEGIN /*Se no possui alquota por tipo de operao*/
               IF NOT (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o maior nivel*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                                         LEFT JOIN PRODUTO P
                                         ON (A.CDPRODUTO = P.CDPRODUTO)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (P.CDARVPROD = @V_CDARVPROD)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA A LEFT JOIN PRODUTO P
                                                                 ON (A.CDPRODUTO = P.CDPRODUTO)
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (P.CDARVPROD = @V_CDARVPROD)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL, NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END
               ELSE IF (@V_CDARVPROD IS NULL) BEGIN /*Se h alquota para o produto "todos"*/
                  DECLARE C_ALIQUOTA CURSOR FOR
                       SELECT A.NRALIQUOTA, VRPERREDBC, VR1PERREDBC, VRPARDEDUZ, VRALIQUOTA,
                              DSMENSALIQU, IDTRATIMPST, VRPBCSUBTRIB, IDPMCBASECAL,
                              CDMENSAGEM, VRADICALIQFECP
                         FROM ALIQUOTA A LEFT JOIN MENSALIQU M
                                         ON (A.NRALIQUOTA = M.NRALIQUOTA)
                        WHERE (A.CDIMPOSTO = @P_IMPOSTO)
                          AND (A.IDENTRSAIDAL = @P_ENTRSAIDAL)
                          AND (A.CDEMPRESA = @P_EMPRESA)
                          AND (A.CDINSCESTA IS NULL)
                          AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                          AND (A.CDPRODUTO IS NULL)
                          AND ((A.CDPAISORIG = @P_PAISORIG)
                          AND (A.SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                          AND ((A.CDPAISDEST = @P_PAISDEST)
                          AND (A.SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                          AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                          AND ((A.VRINIFAIXA < @V_VRITEMPROD)
                          AND (A.VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                          AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                          AND (A.DTINIVALID = (SELECT MAX(DTINIVALID)
                                                 FROM ALIQUOTA
                                                WHERE (CDIMPOSTO = @P_IMPOSTO)
                                                  AND (IDENTRSAIDAL = @P_ENTRSAIDAL)
                                                  AND (CDEMPRESA = @P_EMPRESA)
                                                  AND (CDINSCESTA IS NULL)
                                                  AND (A.CDGRUPOFISC = @V_CDGRUPOFISC)
                                                  AND (CDPRODUTO IS NULL)
                                                  AND ((CDPAISORIG = @P_PAISORIG)
                                                  AND (SGESTAORIG = @P_ESTAORIG) OR @V_UTUFORIG = 'N')
                                                  AND ((CDPAISDEST = @P_PAISDEST)
                                                  AND (SGESTADEST = @P_ESTADEST) OR @V_UTUFDEST = 'N')
                                                  AND ((CDMUNIORIG = @P_MUNIORIG AND @V_UTMUNIORIG = 'O') OR (CDMUNIDEST = @P_MUNIORIG AND @V_UTMUNIORIG = 'D') OR (@V_UTMUNIORIG = 'N'))
                                                  AND ((VRINIFAIXA < @V_VRITEMPROD)
                                                  AND (VRFIMFAIXA >= @V_VRITEMPROD) OR @V_UTFAIXVLR = 'N')
                                                  AND (CHARINDEX(ISNULL(RTRIM(CDTIPOOPER),ISNULL(@P_CDTIPOOPER,'_')),ISNULL(@P_CDTIPOOPER,'_')) = 1 OR @V_IDUTALIQTPOPER = 'N')
                                                  AND (DTINIVALID <= @P_INIVALID)));
                  OPEN C_ALIQUOTA;
                  FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,  @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                  @V_ALIQ_VRPARDEDUZ,  @V_ALIQ_VRALIQUOTA,
                                                  @V_ALIQ_DSMENSALIQU, @V_ALIQ_IDTRATIMPST,
                                                  @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                  @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;

                  WHILE (@@FETCH_STATUS = 0) BEGIN
                     SELECT @V_EXISTE = COUNT(*)
                       FROM ALIQTEMP
                      WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;

                     IF @V_EXISTE > 0 BEGIN
                        DELETE FROM ALIQTEMP
                        WHERE NRALIQTEMP = @P_FILIAL+@P_LANCTONF;
                     END;

                     SELECT @V_COUNTMENS = COUNT(*)
                       FROM MENSAGEM
                      WHERE CDMENSAGEM = @V_ALIQ_CDMENSAGEM
                        AND IDSITUACAOMENS = 'I';

                     IF @V_COUNTMENS > 0 BEGIN
                        SET @V_DSMENSALIQU = NULL;
                     END
                     ELSE BEGIN
                        SET @V_DSMENSALIQU = @V_ALIQ_DSMENSALIQU;
                     END;

                     INSERT INTO ALIQTEMP
                        (NRALIQTEMP, NRSEQUALIQ, VRPREDBCTMP, VRPARDEDTMP,
                         VRALIQUTMP, DSMENSALITMP, IDTRIMPSTTMP, VRPBCSTTMP,
                         IDPMCBSCALTMP, CDINSCESTAATMP, VR1PREDBCTMP, VRADICALIQFECP)
                     VALUES
                        (@P_FILIAL+@P_LANCTONF, @V_SEQUENCIA, @V_ALIQ_VRPERREDBC, @V_ALIQ_VRPARDEDUZ,
                         @V_ALIQ_VRALIQUOTA, @V_DSMENSALIQU, @V_ALIQ_IDTRATIMPST, @V_ALIQ_VRPBCSUBTRIB,
                         @V_ALIQ_IDPMCBASECAL,NULL, @V_ALIQ_VR1PERREDBC, @V_ALIQ_VRADICALIQFECP);

                     FETCH NEXT FROM C_ALIQUOTA INTO @V_ALIQ_NRALIQUOTA,   @V_ALIQ_VRPERREDBC, @V_ALIQ_VR1PERREDBC,
                                                    @V_ALIQ_VRPARDEDUZ,   @V_ALIQ_VRALIQUOTA,
                                                    @V_ALIQ_DSMENSALIQU,  @V_ALIQ_IDTRATIMPST,
                                                    @V_ALIQ_VRPBCSUBTRIB, @V_ALIQ_IDPMCBASECAL,
                                                    @V_ALIQ_CDMENSAGEM, @V_ALIQ_VRADICALIQFECP;
                  END;
                  CLOSE C_ALIQUOTA;
                  DEALLOCATE C_ALIQUOTA;
               END;
            END;
         END;
      END;
   END;   
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CONSULTA_ALIQUOTA','6.05.xxx';
GO
